@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2016 Nullsoft and Contributors
+ * Copyright (C) 1999-2017 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -56,7 +56,7 @@ namespace Apple { // defines struct section
 using namespace std;
 
 extern int g_display_errors;
-extern FILE *g_output;
+extern FILE *g_output, *g_errout;
 
 double my_wtof(const wchar_t *str) 
 {
@@ -70,7 +70,7 @@ size_t my_strncpy(TCHAR*Dest, const TCHAR*Src, size_t cchMax)
   // Dest and Src must be valid, Dest is always \0 terminated.
   // Returns number of TCHARs copied to Dest (not counting \0); min(strlen(Src),cchMax-1).
   size_t cch = 0;
-  if (cchMax) for (TCHAR c; --cchMax;) if (!(c = Src[cch])) break; else Dest[cch++] = c;
+  if (cchMax) for (TCHAR c; --cchMax;) { if (!(c = Src[cch])) break; Dest[cch++] = c; }
   Dest[cch] = _T('\0');
   return cch;
 }
@@ -159,20 +159,20 @@ TCHAR *CharPrev(const TCHAR *s, const TCHAR *p) {
       break;
     s = n;
   }
-  return (TCHAR *) s;
+  return const_cast<TCHAR*>(s);
 }
 
 char *CharNextA(const char *s) {
   int l = 0;
   if (s && *s)
     l = max(1, mblen(s, MB_CUR_MAX));
-  return (char *) s + l;
+  return const_cast<char*>(s + l);
 }
 
 wchar_t *CharNextW(const wchar_t *s) {
-  if (sizeof(*s)==2 && IsLeadSurrogateUTF16(*s)) (wchar_t*) ++s; //BUGBUG: This assumes that 16bit wchar_t == UTF16
-  // else if(...) BUGBUG: Is this the best we can do? What about combining characters/diacritics etc?
-  return (wchar_t*) s + 1;
+  if (sizeof(*s) == 2 && IsLeadSurrogateUTF16(*s)) ++s; //BUGBUG: This assumes that 16bit wchar_t == UTF16
+  // else if (...) BUGBUG: Is this the best we can do? What about combining characters/diacritics etc?
+  return const_cast<wchar_t*>(s + 1);
 }
 
 char *CharNextExA(WORD codepage, const char *s, int flags) {
@@ -182,7 +182,7 @@ char *CharNextExA(WORD codepage, const char *s, int flags) {
   int len = mblen(s, strlen(s));
   if (len > 0) np = s + len; else np = s + 1;
   setlocale(LC_CTYPE, orglocct);
-  return (char *) np;
+  return const_cast<char*>(np);
 }
 
 int wsprintf(TCHAR *s, const TCHAR *format, ...) {
@@ -203,7 +203,7 @@ static char g_nrt_iconv_narrowlocbuf[50], *g_nrt_iconv_narrowloc = 0;
 #ifdef HAVE_LANGINFO_H // BUGBUG: scons needs to check for HAVE_LANGINFO_H and HAVE_NL_LANGINFO support?
 #include <langinfo.h>
 #endif
-bool NSISRT_Initialize()
+bool NSISRT_Initialize() // Init function for POSIX
 {
   iconvdescriptor id;
   g_nrt_iconv_narrowloc = const_cast<char*>(""); // Use "" and not "char", "char" is a GNU extension?
@@ -222,7 +222,7 @@ bool NSISRT_Initialize()
       create_code_page_string(g_nrt_iconv_narrowloc, cchmax, CP_UTF8);
     }
   }
-  return !!nsis_iconv_get_host_endian_ucs4_code();
+  return nsis_iconv_get_host_endian_ucs4_code() && IsValidCodePage(NSISRT_GetASCIICodepage());
 }
 
 const char* NSISRT_setlocale_wincp(int cat, unsigned int cp)
@@ -554,7 +554,7 @@ void PathConvertWinToPosix(TCHAR*p)
 TCHAR *my_convert(const TCHAR *path)
 {
   TCHAR *converted_path = _tcsdup(path);
-  if(!converted_path)
+  if (!converted_path)
   {
     MY_ERROR_MSG(_T("Error: could not allocate memory in my_convert()\n"));
     return 0;
@@ -678,7 +678,7 @@ tstring get_executable_path(const TCHAR* argv0) {
   return tstring(CtoTString(temp_buf));
 #else /* Linux/BSD/POSIX/etc */
   const TCHAR *envpath = _tgetenv(_T("_"));
-  if( envpath != NULL )
+  if (envpath)
     return get_full_path(envpath);
   else {
     char *path = NULL, *pathtmp;
@@ -686,17 +686,17 @@ tstring get_executable_path(const TCHAR* argv0) {
     int nchars;
     while(1){
       pathtmp = (char*)realloc(path,len+1);
-      if( pathtmp == NULL ){
+      if (pathtmp == NULL) {
         free(path);
         return get_full_path(argv0);
       }
       path = pathtmp;
       nchars = readlink("/proc/self/exe", path, len);
-      if( nchars == -1 ){
+      if (nchars == -1) {
         free(path);
         return get_full_path(argv0);
       }
-      if( nchars < (int) len ){
+      if (nchars < (int) len) {
         path[nchars] = '\0';
         tstring result;
         result = CtoTString(path);
@@ -1019,7 +1019,13 @@ bool GetFileSize64(HANDLE hFile, ULARGE_INTEGER &uli)
   uli.LowPart = GetFileSize(hFile, &uli.HighPart);
   return INVALID_FILE_SIZE != uli.LowPart || !GetLastError();
 }
-#endif
+static HANDLE NSISRT_GetConsoleScreenHandle()
+{
+  DWORD cm;
+  HANDLE hCon = GetStdHandle(STD_OUTPUT_HANDLE);
+  return GetConsoleMode(hCon, &cm) ? hCon : GetStdHandle(STD_ERROR_HANDLE);
+}
+#endif //~ _WIN32
 #if defined(_WIN32) && defined(_UNICODE) && defined(MAKENSIS)
 #include <io.h> // for _get_osfhandle
 bool WINAPI WinStdIO_OStreamInit(WINSIO_OSDATA&osd, FILE*strm, WORD cp, int bom)
@@ -1073,17 +1079,23 @@ end:
   strm.Detach();
   return retval;
 }
+static WINSIO_OSDATA*WinStdIO_GetNativeStreamData(FILE*strm)
+{
+  extern WINSIO_OSDATA g_osdata_stdout, g_osdata_stderr;
+  if (g_output == strm) return &g_osdata_stdout;
+  return g_errout == strm ? &g_osdata_stderr : NULL;
+}
 int WINAPI WinStdIO_vfwprintf(FILE*strm, const wchar_t*Fmt, va_list val)
 {
-  if (g_output == strm && Fmt)
+  WINSIO_OSDATA*pOSD;
+  if (Fmt && (pOSD = WinStdIO_GetNativeStreamData(strm)))
   {
-    extern WINSIO_OSDATA g_osdata_stdout;
     ExpandoString<wchar_t, NSIS_MAX_STRLEN> buf;
     errno = ENOMEM;
-    const size_t cchfmt = buf.StrFmt(Fmt, val, false);
+    const size_t cchfmt = buf.StrVFmt(Fmt, val, false);
     UINT cch = (UINT) cchfmt;
     assert(sizeof(size_t) <= 4 || cchfmt == cch);
-    if (cch && !WinStdIO_OStreamWrite(g_osdata_stdout, buf, cch))
+    if (cch && !WinStdIO_OStreamWrite(*pOSD, buf, cch))
     {
       cch = 0, errno = EIO;
     }
@@ -1107,12 +1119,36 @@ int WinStdIO_wprintf(const wchar_t*Fmt, ...)
   va_end(val);
   return rv;
 }
+static HANDLE NSISRT_FastGetConsoleScreenHandle()
+{
+  extern WINSIO_OSDATA g_osdata_stdout, g_osdata_stderr;
+  return WinStdIO_IsConsole(g_osdata_stdout) ? g_osdata_stdout.hNative : g_osdata_stderr.hNative;
+}
+bool NSISRT_Initialize() // Init function for MakeNSIS Win32
+{
+  static bool inited = false;
+  if (inited) return inited;
+  extern WINSIO_OSDATA g_osdata_stdout, g_osdata_stderr;
+  g_osdata_stderr.mode = g_osdata_stdout.mode = 0, g_osdata_stderr.hNative = g_osdata_stdout.hNative = 0;
+  return (inited = true);
+}
+#elif defined(_WIN32)
+#define NSISRT_FastGetConsoleScreenHandle NSISRT_GetConsoleScreenHandle
+bool NSISRT_Initialize() { return true; } // Init function for non-MakeNSIS Win32 (NSISRT_DEFINEGLOBALS sets g_output and g_errout)
 #endif
 
+void PrintColorFmtErrMsg(const TCHAR *fmtstr, va_list args)
+{
+  PrintColorFmtMsg_WARN(_T("")); // flush g_output
+  SetPrintColorERR();
+  _vftprintf(g_errout, fmtstr, args), fflush(g_errout);
+  ResetPrintColor();
+}
+
 void PrintColorFmtMsg(unsigned int type, const TCHAR *fmtstr, va_list args)
 {
 #ifdef _WIN32
-  const HANDLE hWin32Con = GetStdHandle(STD_OUTPUT_HANDLE);
+  HANDLE hWin32Con = NSISRT_FastGetConsoleScreenHandle();
   static INT32 contxtattrbak = -1;
   WORD txtattr = 0;
   if (contxtattrbak < 0)
