@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2016 Nullsoft and Contributors
+ * Copyright (C) 1999-2017 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -34,6 +34,7 @@
 
 extern double my_wtof(const wchar_t *str);
 extern size_t my_strncpy(TCHAR*Dest, const TCHAR*Src, size_t cchMax);
+template<class T> bool strtrycpy(T*Dest, const T*Src, size_t cchCap) { size_t c = my_strncpy(Dest, Src, cchCap); return c < cchCap && !Src[c]; }
 size_t my_strftime(TCHAR *s, size_t max, const TCHAR  *fmt, const struct tm *tm);
 
 // Adds the bitmap in filename using resource editor re as id id.
@@ -77,7 +78,8 @@ public:
     if (!p) throw std::bad_alloc();
     m_heap = (T*) p;
   }
-  size_t StrFmt(const T*FmtStr, va_list Args, bool throwonerr = true)
+  size_t StrFmt(const T*f, ...) { va_list v; va_start(v, f); size_t r = StrVFmt(f, v); va_end(v); return r; }
+  size_t StrVFmt(const T*FmtStr, va_list Args, bool throwonerr = true)
   {
     size_t n = ExpandoStrFmtVaList(m_stack, COUNTOF(m_stack), &m_heap, FmtStr, Args);
     if (throwonerr && !n && *FmtStr) throw std::bad_alloc();
@@ -105,6 +107,8 @@ public:
 
 int sane_system(const TCHAR *command);
 
+#define NSISRT_DEFINEGLOBALS() int g_display_errors=1; FILE *g_output=stdout, *g_errout=stderr
+void PrintColorFmtErrMsg(const TCHAR *fmtstr, va_list args);
 void PrintColorFmtMsg(unsigned int type, const TCHAR *fmtstr, va_list args);
 void FlushOutputAndResetPrintColor();
 #ifdef _WIN32
@@ -161,11 +165,13 @@ inline void PrintColorFmtMsg_ERR(const TCHAR *fmtstr, ...)
 {
   va_list val;
   va_start(val,fmtstr);
-  PrintColorFmtMsg(2, fmtstr, val);
+  PrintColorFmtErrMsg(fmtstr, val);
   va_end(val);
 }
 
 
+bool NSISRT_Initialize();
+#define NSISRT_GetASCIICodepage() ( 1252 )
 #ifndef _WIN32
 // iconv const inconsistency workaround by Alexandre Oliva
 template <typename T>
@@ -233,7 +239,6 @@ BOOL IsValidCodePage(UINT CodePage);
 #else
 #define CharNext CharNextA
 #endif
-bool NSISRT_Initialize();
 #define NSISRT_free(p) ( free((void*)(p)) )
 wchar_t* NSISRT_mbtowc(const char *Str);
 char* NSISRT_wctomb(const wchar_t *Str);
@@ -260,8 +265,6 @@ int my_open(const TCHAR *pathname, int flags);
 
 #else // _WIN32
 
-#define NSISRT_Initialize() (true)
-
 #define my_convert(x) (x)
 #define my_convert_free(x)
 
@@ -276,6 +279,15 @@ FILE* my_fopen(const TCHAR *path, const char *mode);
 // assumption: T is an int type
 template <class T> inline T align_to_512(const T x) { return (x+511) & ~511; }
 
+template<class T> inline T read_ptr_as(const void*p)
+{
+#if defined(_MSC_VER) && (_MSC_VER-0 < 1400) // TODO: Figure out which versions are able to optimize the memcpy
+  return *(T*)(p);
+#else
+  T t; memcpy(&t, p, sizeof(T)); return t;
+#endif
+}
+
 // some values need to be truncated from size_t to [unsigned] int, using this function is better than a plain cast
 template<class R, class T> inline R internaltruncate_cast(T t) {
   assert((~((T)0)) > T(0)); // Only unsigned types supported right now
